/*********
  Pedro Candeias - Force. temperature, humidity sensor board.
  Based on Rui Santos Sketch and taken from  https://RandomNerdTutorials.com/esp32-ble-server-client/
*********/

#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>
#include <BLE2902.h>
// #include <Wire.h>
// #include <Adafruit_Sensor.h>
// #include <Adafruit_BME280.h>

//BLE server name
#define bleServerName "FTH Sensor"

//  Adafruit_BME280 bme; // I2C
// float temp;
// float tempF;
// float hum;

// Force Sensitive Sensor
int fsPinFront = 3;
int fsReadingFront;

// int fsPinMiddle = 4;
// int fsReadingMiddle;

// int fsPinBack = 5;
// int fsReadingBack;

// Timer variables
unsigned long lastTime = 0;
unsigned long timerDelay = 10000;

bool deviceConnected = false;

// Get UUID's from https://www.uuidgenerator.net/
#define SERVICE_UUID "91bad492-b950-4226-aa2b-4ede9fa42f59"

// Force Front Characteristic and Descriptor
BLECharacteristic fthForceFrontCharacteristics("78354994-85b4-4fd9-8111-041dcda92e01", BLECharacteristic::PROPERTY_NOTIFY);
BLEDescriptor fthForceFrontDescriptor(BLEUUID((uint16_t)0x2902));

// Force Middle Characteristic and Descriptor
// BLECharacteristic fthForceMiddleCharacteristics("1de448ff-9842-4171-9f74-ff2ac6bc62cd", BLECharacteristic::PROPERTY_NOTIFY);
// BLEDescriptor fthForceMiddleDescriptor(BLEUUID((uint16_t)0x2902));

// Force Back Characteristic and Descriptor
// BLECharacteristic fthForceBackCharacteristics("c02ead93-9e2e-40c9-a741-f7e253b6a663", BLECharacteristic::PROPERTY_NOTIFY);
// BLEDescriptor fthForceBackDescriptor(BLEUUID((uint16_t)0x2902));

// Temperature Characteristic and Descriptor
// BLECharacteristic fthTemperatureCharacteristics("cba1d466-344c-4be3-ab3f-189f80dd7518", BLECharacteristic::PROPERTY_NOTIFY);
// BLEDescriptor fthTemperatureDescriptor(BLEUUID((uint16_t)0x2902));

// Humidity Characteristic and Descriptor
// BLECharacteristic fthHumidityCharacteristics("ca73b3ba-39f6-4ab3-91ae-186dc9577d99", BLECharacteristic::PROPERTY_NOTIFY);
// BLEDescriptor fthHumidityDescriptor(BLEUUID((uint16_t)0x2903));

//Setup callbacks onConnect and onDisconnect
class MyServerCallbacks: public BLEServerCallbacks {
  void onConnect(BLEServer* pServer) {
    deviceConnected = true;
  };
  void onDisconnect(BLEServer* pServer) {
    deviceConnected = false;
  }
};

// void initBME(){
//   if (!bme.begin(0x76)) {
//     Serial.println("Could not find a valid BME280 sensor, check wiring!");
//     while (1);
//   }
// }

void setup() {
  // Start serial communication 
  Serial.begin(115200);

  // Init BME Sensor
  // initBME();

  // Create the BLE Device
  BLEDevice::init(bleServerName);

  // Create the BLE Server
  BLEServer *pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());

  // Create the BLE Service
  BLEService *fthService = pServer->createService(SERVICE_UUID);

  // Create BLE Characteristics and Create a BLE Descriptor

  // Force
  fthService->addCharacteristic(&fthForceFrontCharacteristics);
  fthForceFrontDescriptor.setValue("Force");
  fthForceFrontCharacteristics.addDescriptor(&fthForceFrontDescriptor);

  // fthService->addCharacteristic(&fthForceMiddleCharacteristics);
  // fthForceMiddleDescriptor.setValue("Force");
  // fthForceMiddleCharacteristics.addDescriptor(&fthForceMiddleDescriptor);

  // fthService->addCharacteristic(&fthForceBackCharacteristics);
  // fthForceBackDescriptor.setValue("Force");
  // fthForceBackCharacteristics.addDescriptor(&fthForceBackDescriptor);

  // Temperature
  //fthService->addCharacteristic(&fthTemperatureCharacteristics);
  //fthTemperatureDescriptor.setValue("Temperature ");
  //fthTemperatureCharacteristics.addDescriptor(&fthTemperatureDescriptor);

  // Humidity
  //fthService->addCharacteristic(&fthHumidityCharacteristics);
  //fthHumidityDescriptor.setValue("Humidity");
  //fthHumidityCharacteristics.addDescriptor(new BLE2902());
  
  // Start the service
  fthService->start();

  // Start advertising
  BLEAdvertising *pAdvertising = BLEDevice::getAdvertising();
  pAdvertising->addServiceUUID(SERVICE_UUID);
  pServer->getAdvertising()->start();
  Serial.println("Waiting a client connection to notify...");
}

void loop() {
  if (deviceConnected) {

    if ((millis() - lastTime) > timerDelay) {

      fsReadingFront = analogRead(fsPinFront);
      //fsReadingMiddle = analogRead(fsPinMiddle);
      //fsReadingBack = analogRead(fsPinBack);
      
      //Set Force Front Characteristic value and notify connected client
      static char forceFrontChar[2];
      dtostrf(fsReadingFront, 2, 0, forceFrontChar);
      fthForceFrontCharacteristics.setValue(forceFrontChar);
      fthForceFrontCharacteristics.notify();
      Serial.print("normal output:");
      Serial.println(fsReadingFront);
      Serial.print("char output:");
      Serial.println(forceFrontChar);
      //Set Force Middle Characteristic value and notify connected client
      //fthForceMiddleCharacteristics.setValue(fsReadingMiddle);
      //fthForceMiddleCharacteristics.notify();
    
      
      //Set Force Back Characteristic value and notify connected client
      //fthForceBackCharacteristics.setValue(fsReadingBack);
      //fthForceBackCharacteristics.notify();
      
      //Notify temperature reading from BME sensor
    
      // Read temperature
      // temp = bme.readTemperature();
      // Read humidity
      // hum = bme.readHumidity();

      //static char temperatureCTemp[6];
      //dtostrf(temp, 6, 2, temperatureCTemp);
      //Set temperature Characteristic value and notify connected client
      //fthTemperatureCharacteristics.setValue(temperatureCTemp);
      //fthTemperatureCharacteristics.notify();
      //Serial.print("Temperature : ");
      //Serial.print(temp);
      //Serial.print(" ºC");
      
      //Notify humidity reading from BME
      //static char humidityTemp[6];
      //dtostrf(hum, 6, 2, humidityTemp);
      //Set humidity Characteristic value and notify connected client
      //fthHumidityCharacteristics.setValue(humidityTemp);
      //fthHumidityCharacteristics.notify();   
      //Serial.print(" - Humidity: ");
      //Serial.print(hum);
      //Serial.println(" %");
      
      lastTime = millis();
    }
  } 
}
